package org.apidesign.apifest08.currency;

import java.util.Currency;


/** 
 * Converts one currency to other. The conversion is unidirectional. 
 * For example you can have convertor that converts USD (sourceCurrency) to CZK (destination currency). You can call the {@link Convertor#convert(Money)} method
 * with amount in USD to get the equivalent in CZK. If you need convert CZK to USD you can call {@link Convertor#revert()} method to get CZK to USD 
 * convertor. To create a convertor instance call {@link ConvertorFactory#createConvertor(Currency, Currency)}.
 */
public interface Convertor {
	/**
	 * Converts amount in source currency to amount in destination currency. The result is rounded to two decimal places.
	 * @param money 
	 * @return
	 * @throws IllegalArgumentException if money.getCurrency is not equal to sourceCurrency.
	 */
	public Money convert(Money money);

	/**
	 * Returns convertor that converts from destination currency to source currency with the same exchange rate. 
	 * @return
	 */
	public Convertor revert();
	
	/**
	 * Returns source currency.
	 * @return
	 */
	public Currency getSourceCurrency();

	/**
	 * Returns destination currency.
	 * @return
	 */
	public Currency getDestinationCurrency();
}
