package org.apidesign.apifest08.test;

import java.util.Currency;

import junit.framework.TestCase;

import org.apidesign.apifest08.currency.Convertor;
import org.apidesign.apifest08.currency.ConvertorFactory;
import org.apidesign.apifest08.currency.MoneyImpl;

/** Finish the Convertor API, and then write bodies of methods inside
 * of this class to match the given tasks. To fullfil your task, use the
 * API define in the <code>org.apidesign.apifest08.currency</code> package.
 * Do not you reflection, or other hacks as your code
 * shall run without any runtime permissions.
 */
public class Task1Test extends TestCase {
	public static final Currency USD = Currency.getInstance("USD");
	public static final Currency CZK = Currency.getInstance("CZK");
	public static final Currency SKK = Currency.getInstance("SKK");
    public Task1Test(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
    }

    @Override
    protected void tearDown() throws Exception {
    }

    /** Create convertor that understands two currencies, CZK and
     *  USD. Make 1 USD == 17 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     *
     * @return prepared convertor ready for converting USD to CZK and CZK to USD
     */
    public static Convertor createCZKtoUSD() {
      //You can use both variants. I wrote the first one but than I realized that maybe
      //you want me to write the second one. So I have written both. 
      return ConvertorFactory.createConvertor(CZK, USD);
      //return ConvertorFactory.createConvertor(new MoneyImpl(17,CZK), new MoneyImpl(1,USD));
    }

    /** Create convertor that understands two currencies, CZK and
     *  SKK. Make 100 SKK == 80 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     * 
     * @return prepared convertor ready for converting SKK to CZK and CZK to SKK
     */
    public static Convertor createSKKtoCZK() {
    	//You can use both variants. I wrote the first one but than I realized that maybe
        //you want me to write the second one. So I have written both.
        //return ConvertorFactory.createConvertor(SKK, CZK);
        return ConvertorFactory.createConvertor(new MoneyImpl(100,SKK), new MoneyImpl(80,CZK));
    }
    
    /** Use the convertor from <code>createCZKtoUSD</code> method and do few conversions
     * with it.
     */
    public void testCurrencyCZKUSD() throws Exception {
        Convertor czkToUsdConvertor = createCZKtoUSD();
        // convert $5 to CZK using c:
        Convertor usdToCzkConvertor = czkToUsdConvertor.revert();
		assertEquals("Result is 85 CZK",new MoneyImpl(85,CZK), usdToCzkConvertor.convert(new MoneyImpl(5,USD)));

        // convert $8 to CZK
        assertEquals("Result is 136 CZK",new MoneyImpl(136,CZK), usdToCzkConvertor.convert(new MoneyImpl(8,USD)));

        // convert 1003CZK to USD
        assertEquals("Result is 59 USD", new MoneyImpl(59,USD), czkToUsdConvertor.convert(new MoneyImpl(1003,CZK)));
    }

    /** Use the convertor from <code>createSKKtoCZK</code> method and do few conversions
     * with it.
     */
    public void testCurrencySKKCZK() throws Exception {
        Convertor skkToCzkConvertor = createSKKtoCZK();
        // convert 16CZK using c:
        assertEquals("Result is 20 SKK", new MoneyImpl(20,SKK), skkToCzkConvertor.revert().convert(new MoneyImpl(16,CZK)));

        // convert 500SKK to CZK
        assertEquals("Result is 400 CZK", new MoneyImpl(400,CZK), skkToCzkConvertor.convert(new MoneyImpl(500,SKK)));
    }
}

