package org.apidesign.apifest08.currency;

import static org.apidesign.apifest08.currency.Assert.notNull;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Currency;
import java.util.List;

/**
 * The factory for {@link Convertor}.
 * 
 * @see #newInstance()
 */
public final class ConvertorFactory {
	//do not expose constructor
	private ConvertorFactory(){}
	
	/**
	 * @return a convertor instance. 
	 *  
	 * @throws CurrencyException in a convertor cannot be instantiated
	 */
	public static Convertor newInstance() throws CurrencyException{
		return new DefaultConvertor();
	}
	
	private static final class DefaultConvertor extends  Convertor {
		private List<Bid> bids;
		
		private DefaultConvertor() {
			bids = new ArrayList<Bid>();
			Bid usdCzk = new Bid(Currencies.USD, Currencies.CZK, new BigDecimal("17"));
			bids.add(usdCzk);
			Bid skCzk = new Bid(Currencies.SKK, Currencies.CZK, new BigDecimal("0.8"));
			bids.add(skCzk);
		}
		
		@Override
		public Amount convert(Amount from, Currency targetCurrency)
				throws CurrencyException {
			return convert(from.getValue(), from.getCurrency(), targetCurrency);
			
		}

		@Override
		public Amount convert(BigDecimal amount, Currency from,
				Currency targetCurrency) throws ConversionException,
				UnsupportedConversionException {
			notNull(from, "from");
			notNull(targetCurrency, "targetCurrency");
			
			int index = bids.indexOf(new Bid(from, targetCurrency));
			if(index == -1) {
				throw new UnsupportedConversionException(from, targetCurrency);
			}
			Bid bid = bids.get(index);
			BigDecimal bidValue = bid.getBidValue(from, targetCurrency);
			BigDecimal result = bidValue.multiply(amount);			
			return new Amount(result, targetCurrency);
		}

   }
}
